import tempfile

from moto import mock_s3

from caretaker.backend.abstract_backend import StoreOutcome
from caretaker.tests.utils import upload_temporary_file
from caretaker.tests.frontend.django.backend.s3.caretaker_test import \
    AbstractDjangoS3Test


@mock_s3
class TestPushBackupDjangoS3(AbstractDjangoS3Test):
    def setUp(self):
        self.logger.info('Setup for push_backup')

        self.create_bucket()

    def tearDown(self):
        self.logger.info('Teardown for push_backup')
        pass

    def test(self):
        self.logger.info('Testing push_backup')

        with tempfile.TemporaryDirectory() as temporary_directory_name:

            # set up a temporary file
            result, temporary_file = upload_temporary_file(
                test_class=self,
                temporary_directory_name=temporary_directory_name,
                contents=self.test_contents)

            self.assertTrue(result == StoreOutcome.STORED)

            # run a second time and should not store the result
            result, temporary_file = upload_temporary_file(
                test_class=self,
                temporary_directory_name=temporary_directory_name,
                contents=self.test_contents)

            self.assertTrue(result == StoreOutcome.IDENTICAL)

            # now test that when we add a new file it versions it
            with temporary_file.open('w') as out_file:
                out_file.write('test2')

            result = self.frontend.push_backup(
                backup_local_file=temporary_file, remote_key=self.json_key,
                backend=self.backend, bucket_name=self.bucket_name)

            self.assertTrue(result == StoreOutcome.STORED)



import tempfile
from pathlib import Path

from moto import mock_s3

from caretaker.utils import file
from caretaker.backend.abstract_backend import StoreOutcome
from caretaker.tests.frontend.django.backend.s3.caretaker_test import \
    AbstractDjangoS3Test
from caretaker.tests.utils import upload_temporary_file, \
    file_in_zip
from caretaker.utils.zip import create_zip_file


@mock_s3
class TestPullBackupDjangoS3(AbstractDjangoS3Test):
    def setUp(self):
        self.logger.info('Setup for pull_backup S3')

        self.create_bucket()

    def tearDown(self):
        self.logger.info('Teardown for pull_backup S3')
        pass

    def test(self):
        self.logger.info('Testing pull_backup S3')
        with tempfile.TemporaryDirectory() as temporary_directory_name:

            temporary_directory_name = file.normalize_path(
                temporary_directory_name)

            # set up a temporary file
            result, temporary_file = upload_temporary_file(
                test_class=self,
                temporary_directory_name=str(temporary_directory_name),
                contents=self.test_contents)

            self.assertTrue(result == StoreOutcome.STORED)

            # list the results to get a versionId
            result = self.frontend.list_backups(
                remote_key=self.json_key, bucket_name=self.bucket_name,
                backend=self.backend
            )

            download_location = temporary_directory_name / self.json_key

            result = self.frontend.pull_backup(
                backup_version=result[0]['version_id'],
                remote_key=self.json_key,
                bucket_name=self.bucket_name,
                backend=self.backend,
                out_file=download_location
            )

            self.assertIsNotNone(result)
            self.assertTrue(download_location.exists())

            with download_location.open('r') as in_file:
                result = in_file.read()

                self.assertTrue(result == 'test')

            # test that we can push and pull a binary file
            zip_path = Path(temporary_directory_name) / self.data_key

            zip_file = create_zip_file(
                input_paths=[temporary_directory_name],
                output_file=Path(zip_path)
            )

            self.assertTrue(zip_file.exists())

            # upload the file
            self.frontend.push_backup(
                backup_local_file=zip_file, remote_key=self.data_key,
                backend=self.backend, bucket_name=self.bucket_name)

            # delete the zip file locally
            zip_file.unlink(missing_ok=True)
            self.assertFalse(zip_file.exists())

            # list the results to get a versionId
            result = self.frontend.list_backups(
                remote_key=self.data_key, bucket_name=self.bucket_name,
                backend=self.backend
            )

            data_download_location = temporary_directory_name / self.json_key

            # pull it back down
            self.frontend.pull_backup(
                backup_version=result[0]['version_id'],
                remote_key=self.data_key,
                bucket_name=self.bucket_name,
                backend=self.backend,
                out_file=data_download_location
            )

            self.assertTrue(file_in_zip(
                zip_file=data_download_location, filename=self.json_key
            ))



import logging

import django
from django.test import TestCase
from testfixtures import log_capture

from caretaker.utils import log


class LoggerDefaultTestCase(TestCase):
    @log_capture()
    def test_root(self, captures):
        logger = log.get_logger(level=logging.DEBUG)
        logger.debug("a noise")
        logger.info("a message")
        logger.warning("a warning")
        logger.error("an error")
        logger.critical("a critical error")
        captures.check(
            ("root", "DEBUG", "a noise"),
            ("root", "INFO", "a message"),
            ("root", "WARNING", "a warning"),
            ("root", "ERROR", "an error"),
            ("root", "CRITICAL", "a critical error"),
        )

    @log_capture()
    def test_django(self, captures):
        logger = log.get_logger('django')
        logger.debug("a noise")
        logger.info("a message")
        logger.warning("a warning")
        logger.error("an error")
        logger.critical("a critical error")
        if django.VERSION < (1, 9):
            captures.check(
                ("django", "DEBUG", "a noise"),
                ("django", "INFO", "a message"),
                ("django", "WARNING", "a warning"),
                ("django", "ERROR", "an error"),
                ("django", "CRITICAL", "a critical error"),
            )
        else:
            captures.check(
                ("django", "INFO", "a message"),
                ("django", "WARNING", "a warning"),
                ("django", "ERROR", "an error"),
                ("django", "CRITICAL", "a critical error"),
            )

    @log_capture()
    def test_caretaker(self, captures):
        logger = log.get_logger('caretaker')
        logger.debug("a noise")
        logger.info("a message")
        logger.warning("a warning")
        logger.error("an error")
        logger.critical("a critical error")
        captures.check(
            ("caretaker", "INFO", "a message"),
            ("caretaker", "WARNING", "a warning"),
            ("caretaker", "ERROR", "an error"),
            ("caretaker", "CRITICAL", "a critical error"),
        )



import json
import tempfile
from pathlib import Path

import django
from moto import mock_s3

from caretaker.utils import file
from caretaker.backend.abstract_backend import StoreOutcome
from caretaker.tests.frontend.django.backend.s3.caretaker_test import \
    AbstractDjangoS3Test
from caretaker.tests.utils import upload_temporary_file, file_in_zip


@mock_s3
class TestRunBackupDjangoS3(AbstractDjangoS3Test):
    def setUp(self):
        self.logger.info('Setup for run_backup S3')

        self.create_bucket()

        django.setup()

    def tearDown(self):
        self.logger.info('Teardown for run_backup S3')
        pass

    def test(self):
        self.logger.info('Testing run_backup S3')

        with tempfile.TemporaryDirectory() as temporary_directory_name:
            temporary_directory_name = file.normalize_path(
                temporary_directory_name)

            # set up a temporary file
            result, temporary_file = upload_temporary_file(
                test_class=self,
                temporary_directory_name=str(temporary_directory_name),
                contents=self.test_contents)

            self.assertTrue(result == StoreOutcome.STORED)

            # create a backup record including this directory
            self.frontend.run_backup(
                output_directory=temporary_directory_name,
                path_list=[temporary_directory_name],
                bucket_name=self.bucket_name,
                backend=self.backend
            )

            # now check that the files exist in S3
            # list the results to get a versionId of the SQL backup
            result = self.frontend.list_backups(
                remote_key=self.dump_key, bucket_name=self.bucket_name,
                backend=self.backend
            )

            Path(temporary_directory_name / self.json_key).unlink(
                missing_ok=True)
            download_location = temporary_directory_name / self.json_key

            result = self.frontend.pull_backup(
                backup_version=result[0]['version_id'],
                remote_key=self.dump_key,
                bucket_name=self.bucket_name,
                backend=self.backend,
                out_file=download_location
            )

            with result.open('r') as in_file:
                response = in_file.read()

                try:
                    json.loads(response)
                except json.JSONDecodeError:
                    self.fail('The stored object could not be JSON decoded')

            # now check the archive zip
            result = self.frontend.list_backups(
                remote_key=self.data_key, bucket_name=self.bucket_name,
                backend=self.backend
            )

            Path(temporary_directory_name / self.data_key).unlink(
                missing_ok=True)
            download_location = temporary_directory_name / self.data_key

            result = self.frontend.pull_backup(
                backup_version=result[0]['version_id'],
                remote_key=self.data_key,
                bucket_name=self.bucket_name,
                backend=self.backend,
                out_file=download_location
            )

            self.assertTrue(file_in_zip(zip_file=result,
                                        filename=self.json_key))



from django.urls import path

from caretaker import views

urlpatterns = [
    path('list/', views.list_backups, name='list_backups'),
    path('download/<str:backup_type>/version/<str:version_id>/',
         views.download_backup, name='download_backup'),
]



import logging

from django.test import TestCase
import abc

from moto import mock_s3

from caretaker.utils import log


@mock_s3
class AbstractCaretakerTest(TestCase, metaclass=abc.ABCMeta):
    json_key = 'test.json'
    dump_key = 'data.json'
    data_key = 'media.zip'
    test_contents = 'test'
    bucket_name = ''

    logger = None
    backend = None
    frontend = None

    def __init__(self, logger: logging.Logger | None = None,
                 method_name='test'):
        super().__init__(methodName=method_name)
        self.logger = logger

        self.logger = log.get_logger('caretaker')



import djclick as click

from caretaker.frontend.abstract_frontend import FrontendFactory, \
    FrontendNotFoundError

from caretaker.utils import log


@click.command()
@click.argument('output-directory')
@click.option('--additional-files', '-a', multiple=True,
              help='Additional directories to add to the zip file',
              type=str)
@click.option('--frontend-name', '-f',
              help='The name of the frontend to use',
              type=str)
def command(output_directory: str, additional_files: tuple,
            frontend_name: str = '') -> None:
    """
    Create a local backup archive in the specified OUTPUT-DIRECTORY
    """
    logger = log.get_logger('caretaker')

    try:
        frontend = FrontendFactory.get_frontend(frontend_name=frontend_name,
                                                raise_on_none=True)

        frontend.create_backup(output_directory=output_directory,
                               path_list=list(additional_files))
    except FrontendNotFoundError:
        logger.error('Unable to find a valid frontend')



import abc
import logging

import boto3

from caretaker.backend.abstract_backend import BackendFactory
from caretaker.frontend.abstract_frontend import FrontendFactory
from caretaker.tests.caretaker_main_test import AbstractCaretakerTest
from caretaker.utils import log


class AbstractDjangoS3Test(AbstractCaretakerTest, metaclass=abc.ABCMeta):
    def __init__(self, logger: logging.Logger | None = None):
        super().__init__(logger=logger, method_name='test')

        self.backend = BackendFactory.get_backend('Amazon S3')
        self.frontend = FrontendFactory.get_frontend('Django')

        self.json_key = 'test.json'
        self.dump_key = 'data.json'
        self.data_key = 'media.zip'
        self.test_contents = 'test'

        self.logger = log.get_logger('caretaker')

    def create_bucket(self):
        """
        Sets up a test bucket for a Django frontend, S3 backend scenario

        :return: None
        """
        self.backend.s3 = boto3.client(
            's3',
            region_name='us-east-1',
            aws_access_key_id='fake_access_key',
            aws_secret_access_key='fake_secret_key',
        )

        self.bucket_name = 'a_test_bucket'
        self.backend.s3.create_bucket(Bucket=self.bucket_name)

        self.backend.s3.put_bucket_versioning(
            Bucket=self.bucket_name,
            ChecksumAlgorithm='CRC32',
            VersioningConfiguration={
                'Status': 'Enabled'
            }
        )



import djclick as click

from caretaker.backend.abstract_backend import BackendNotFoundError
from caretaker.frontend.abstract_frontend import FrontendNotFoundError
from caretaker.frontend.abstract_frontend import FrontendFactory
from caretaker.utils import log


@click.command()
@click.argument('output-directory')
@click.option('--backend-name', '-b',
              help='The name of the backend to use',
              type=str)
@click.option('--frontend-name', '-f',
              help='The name of the frontend to use',
              type=str)
def command(output_directory: str, backend_name: str, frontend_name: str) \
        -> None:
    """
    Output terraform files to the specified OUTPUT-DIRECTORY
    """
    logger = log.get_logger('caretaker')

    try:
        frontend, backend = FrontendFactory.get_frontend_and_backend(
            backend_name=backend_name,
            frontend_name=frontend_name,
            raise_on_none=True
        )

        frontend.generate_terraform(
            output_directory=output_directory,
            backend=backend
        )
    except BackendNotFoundError:
        logger.error('Unable to find a valid backend')
    except FrontendNotFoundError:
        logger.error('Unable to find a valid frontend')



import tempfile

from moto import mock_s3

from caretaker.tests.frontend.django.backend.s3.caretaker_test import \
    AbstractDjangoS3Test
from caretaker.tests.utils import upload_temporary_file


@mock_s3
class TestListBackupsDjangoS3(AbstractDjangoS3Test):
    def setUp(self):
        self.logger.info('Setup list_backups S3')

        self.create_bucket()

    def tearDown(self):
        self.logger.info('Teardown list_backups S3')
        pass

    def test(self):
        self.logger.info('Testing list_backups S3')
        with tempfile.TemporaryDirectory() as temporary_directory_name:

            # first test that we get nothing back
            result = self.frontend.list_backups(
                remote_key=self.json_key, bucket_name=self.bucket_name,
                backend=self.backend
            )

            self.assertListEqual(result, [])

            # run the first time to store the result
            upload_temporary_file(
                test_class=self,
                temporary_directory_name=temporary_directory_name,
                contents=self.test_contents)

            # Now check that we get a result
            result = self.frontend.list_backups(
                remote_key=self.json_key, bucket_name=self.bucket_name,
                backend=self.backend
            )

            self.assertIsNotNone(result)
            version = None

            if result:
                self.assertTrue(result[0]['size'] == 4)
                version = result[0]['version_id']

            # now test that when we add a new file it versions it
            result, temporary_file = upload_temporary_file(
                test_class=self,
                temporary_directory_name=temporary_directory_name,
                contents='test2')

            # first test that we get nothing back
            result = self.frontend.list_backups(
                remote_key=self.json_key, bucket_name=self.bucket_name,
                backend=self.backend
            )

            self.assertIsNotNone(result)

            if result:
                self.assertTrue(result[0]['size'] == 5)
                self.assertFalse(result[0]['version_id'] == 'null')
                self.assertFalse(version == result[0]['version_id'])
                version = result[0]['version_id']

            # now run a final time with the same version and check that
            # the latest version is the same
            self.frontend.push_backup(
                backup_local_file=temporary_file, remote_key=self.json_key,
                backend=self.backend, bucket_name=self.bucket_name)

            result = self.frontend.list_backups(
                remote_key=self.json_key, bucket_name=self.bucket_name,
                backend=self.backend
            )

            self.assertIsNotNone(result)

            if result:
                self.assertTrue(
                    result[0]['version_id'] == version)



import filecmp
import importlib
import io
import logging
import tempfile
from pathlib import Path
from types import ModuleType

import boto3
import botocore.exceptions
from django.conf import settings

from caretaker.utils import log
from caretaker.backend.abstract_backend import AbstractBackend, StoreOutcome


def get_backend():
    return S3Backend()


class S3Backend(AbstractBackend):
    @property
    def terraform_files(self) -> list[str]:
        return ['main.tf', 'output.tf']

    @property
    def terraform_template_module(self) -> ModuleType:
        """
        The directory that stores this backend's templates

        :return: a pathlib.Path to the backend's templates'
        """
        return importlib.import_module(
            'caretaker.backend.backends.terraform_aws')

    def __init__(self, logger: logging.Logger | None = None):
        super().__init__(logger)

        self.logger = log.get_logger('caretaker-amazon-s3')

        self.s3 = boto3.client(
            's3',
            aws_access_key_id=settings.AWS_ACCESS_KEY_ID,
            aws_secret_access_key=settings.AWS_SECRET_ACCESS_KEY)

    @property
    def backend_name(self) -> str:
        """
        The display name of the backend

        :return: a string of the backend name
        """
        return 'Amazon S3'

    def versions(self, bucket_name: str, remote_key: str = '') -> list[dict]:
        """
        List the versions of an object in an S3 bucket

        :param remote_key: the remote key (filename) to list
        :param bucket_name: the remote bucket name
        :return: a list of dictionaries containing 'version_id', 'last_modified', and 'size'
        """
        try:
            versions = self.s3.list_object_versions(Bucket=bucket_name,
                                                    Prefix=remote_key)

            if versions and 'Versions' in versions:
                final_versions = [
                    {'version_id': item['VersionId'],
                     'last_modified': item['LastModified'],
                     'size': item['Size']
                     } for item in versions['Versions']
                ]
                return final_versions
            else:
                return []
        except botocore.exceptions.ClientError as ce:
            self.logger.error(
                'Unable to retrieve version list of {} from {} in {} '
                '({})'.format(remote_key, bucket_name, self.backend_name, ce)
            )
            return []

    def store_object(self, local_file: Path, bucket_name: str,
                     remote_key: str, check_identical: bool) -> StoreOutcome:
        """
        Store an object remotely

        :param local_file: the local file to store
        :param bucket_name: the remote bucket name
        :param remote_key: the remote key (filename) of the object
        :param check_identical: whether to check if the last version is already the same as this version
        :return: a response enum StoreOutcome
        """

        if check_identical:
            # download the latest version of the backup to see if it's the same
            # as the local file
            try:
                with tempfile.TemporaryDirectory() as tmp:
                    path = Path(tmp) / 'latest'

                    self.s3.download_file(Filename=str(path),
                                          Bucket=bucket_name,
                                          Key=remote_key)

                    # byte-by-byte comparison
                    # may be slow on big files
                    if filecmp.cmp(path, local_file):
                        self.logger.info(
                            'Latest backup is equal to remote S3 version')
                        return StoreOutcome.IDENTICAL

            except botocore.exceptions.ClientError:
                self.logger.debug('There was a problem comparing the previous '
                                  'version of this object with the stored '
                                  'version. This is not a fatal error and '
                                  'can be caused by this being the first '
                                  'stored version of an object.')

        try:
            # upload the latest version to S3
            self.s3.upload_file(Filename=str(local_file),
                                Bucket=bucket_name, Key=remote_key)

            self.logger.info('Backup {} stored as {}'.format(
                local_file, remote_key))
        except botocore.exceptions.ClientError:
            self.logger.error('There was a problem storing the backup.')
            return StoreOutcome.FAILED

        return StoreOutcome.STORED

    def get_object(self, bucket_name: str, remote_key: str,
                   version_id: str) -> io.BytesIO | None:
        """
        Retrieve an object from the remote store as bytes

        :param bucket_name: the remote bucket name
        :param remote_key: the remote key (filename) of the object
        :param version_id: the version ID to fetch
        :return: the bytes of the retrieved object
        """
        try:
            self.logger.info('Fetching version {} of {}'.format(
                version_id,
                remote_key
            ))

            response_object = io.BytesIO()

            self.s3.download_fileobj(Bucket=settings.CARETAKER_BACKUP_BUCKET,
                                     Key=remote_key,
                                     Fileobj=response_object,
                                     ExtraArgs={'VersionId': version_id})

            response_object.seek(0)

            return response_object

        except botocore.exceptions.ClientError:
            self.logger.error('Unable to download version {} of '
                              '{}'.format(version_id, remote_key))
            return None

    def download_object(self, local_file: Path, bucket_name: str,
                        remote_key: str, version_id: str) -> bool:
        """
        Retrieve an object from the remote store and save it to a file

        :param local_file: the location to store the local file
        :param bucket_name: the remote bucket name
        :param remote_key: the remote key (filename) of the object
        :param version_id: the version ID to fetch
        :return: a true/false boolean of success
        """

        # normalize path
        out_file = Path(local_file).expanduser()

        try:
            self.s3.download_file(Filename=str(out_file),
                                  Bucket=bucket_name,
                                  Key=remote_key,
                                  ExtraArgs={'VersionId': version_id})

            self.logger.info('Saved version {} of {} to {}'.format(
                version_id,
                remote_key,
                out_file
            ))

            return True
        except botocore.exceptions.ClientError:
            self.logger.error('Unable to download version {} of '
                              '{} to {}'.format(version_id, remote_key,
                                                out_file))
            return False



import zipfile
from pathlib import Path

from caretaker.backend.abstract_backend import StoreOutcome
from caretaker.tests.caretaker_main_test import AbstractCaretakerTest
from caretaker.utils import file


def upload_temporary_file(test_class: AbstractCaretakerTest,
                          temporary_directory_name: str,
                          contents: str) -> (StoreOutcome, Path):
    """
    Create a temporary file and upload it to the mocked backend

    :param test_class: the test case in question
    :param temporary_directory_name: the output directory to use
    :param contents: the contents to write to the file
    :return: a 2-tuple of StoreOutcome and pathlib.Path to the file
    """

    temporary_file = file.normalize_path(
        temporary_directory_name) / test_class.json_key

    with temporary_file.open('w') as out_file:
        out_file.write(contents)

    # run the first time to store the result
    result = test_class.frontend.push_backup(
        backup_local_file=temporary_file, remote_key=test_class.json_key,
        backend=test_class.backend, bucket_name=test_class.bucket_name)

    return result, temporary_file


def file_in_zip(zip_file: str, filename: str) -> bool:
    """
    Test whether a file exists inside a zip

    :param zip_file: the input zip file
    :param filename: the file to check for
    :return: True if found, else False
    """
    zip_file = file.normalize_path(zip_file)

    with zipfile.ZipFile(zip_file, 'r') as zf:
        name_list = [filename.split('/')[-1]
                     for filename in zf.namelist()]
        return filename in name_list



import os

from crontab import CronTab, CronItem
from django.conf import settings

from caretaker.utils import log, file

import djclick as click


@click.command()
@click.argument('action', default='')
def command(action: str) -> None:
    """
    Installs crontab entries to run at 00:15 every day
    """
    install_cron(job_name=settings.CARETAKER_BACKUP_BUCKET,
                 action=action, base_dir=settings.BASE_DIR)


def find_job(tab: CronTab, comment: str) -> CronItem | None:
    """
    Locates an existing crontab entry

    :param tab: the Crontab object
    :param comment: the comment attached to the job
    :return: a CrontabItem or None
    """
    for job in tab:
        if job.comment == comment:
            return job
    return None


def install_cron(job_name: str, action: str, base_dir: str) \
        -> CronTab | None:
    """
    Installs a crontab entry to run the backup daily

    :param job_name: the name of the cron job
    :param base_dir: the working directory from which to operate
    :param action: the action to take ("test", "quiet", or ""). "Test" will perform a dry run. Quiet will exit silently with changes unsaved. An empty string will save the crontab file.
    :return:
    """
    logger = log.get_logger('caretaker-cron')
    tab = CronTab(user=True)
    virtualenv = os.environ.get('VIRTUAL_ENV', None)

    base_dir = file.normalize_path(base_dir)

    jobs = [
        {
            'name': 'caretaker_sync_{}_job'.format(job_name),
            'task': 'run_backup',
        },
    ]

    for job in jobs:
        current_job = find_job(tab, job['name'])

        if not current_job:
            django_command = "{0}/manage.py {1}".format(str(base_dir),
                                                        job['task'])
            cron_command = '%s/bin/python3 %s' % (virtualenv, django_command)

            cron_job = tab.new(cron_command, comment=job['name'])
            cron_job.setall("15 0 * * *")

        else:
            logger.info("{name} cron job already exists.".format(
                name=job['name']))

    if action == 'test':
        logger.info(tab.render())
    else:
        tab.write()

    return tab



"""
Django settings for djangoCaretaker project.

Generated by 'django-admin startproject' using Django 4.0.5.

For more information on this file, see
https://docs.djangoproject.com/en/4.0/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/4.0/ref/settings/
"""
import os
from pathlib import Path

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/4.0/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'django-insecure-kr&#=+3^qxrrwa=#sa+ops%+ub!fj-(z!0f0s%ky#0n3dh=7u5'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',

    'caretaker'
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'caretaker.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [BASE_DIR / 'templates']
        ,
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'djangoCaretaker.wsgi.application'


# Database
# https://docs.djangoproject.com/en/4.0/ref/settings/#databases

SYSTEM_ENV = os.environ.get('SYSTEM_ENV', None)
if SYSTEM_ENV == 'PRODUCTION' or SYSTEM_ENV == 'STAGING':
    DEBUG = False
    SECRET_KEY = os.environ.get('SECRET_KEY', None)
    CSRF_COOKIE_SECURE = True

elif SYSTEM_ENV == 'GITHUB_WORKFLOW':
    DEBUG = True
    SECRET_KEY = 'TESTING_KEY_123'
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.postgresql',
            'NAME': 'github_actions',
            'USER': 'postgres',
            'PASSWORD': 'postgres',
            'HOST': '127.0.0.1',
            'PORT': '5432',
        }
    }
elif SYSTEM_ENV == 'DEVELOPMENT':
    DEBUG = True

    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': BASE_DIR / 'db.sqlite3',
        }
    }


# Password validation
# https://docs.djangoproject.com/en/4.0/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/4.0/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/4.0/howto/static-files/

STATIC_URL = 'static/'

# Default primary key field type
# https://docs.djangoproject.com/en/4.0/ref/settings/#default-auto-field

DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'

# AWS backup IAM
AWS_ACCESS_KEY_ID = 'DEFAULT'
AWS_SECRET_ACCESS_KEY = 'DEFAULT'
AWS_DEFAULT_REGION = 'us-east-1'

CARETAKER_BACKUP_BUCKET = 'caretakertestbackup'
CARETAKER_ADDITIONAL_BACKUP_PATHS = ['/home/martin/data', '/home/martin/Pics']
CARETAKER_BACKEND = 'Amazon S3'
CARETAKER_BACKENDS = ['caretaker.backend.backends.s3']
CARETAKER_FRONTEND = 'Django'
CARETAKER_FRONTENDS = ['caretaker.frontend.frontends.django']




import abc
import importlib
import logging
import sys
from enum import Enum
from pathlib import Path
from types import ModuleType

from django.conf import settings


class StoreOutcome(Enum):
    FAILED = 0
    STORED = 1
    IDENTICAL = 2


class AbstractBackend(metaclass=abc.ABCMeta):

    @abc.abstractmethod
    def __init__(self, logger: logging.Logger | None = None):
        self.logger = logger

    @property
    @abc.abstractmethod
    def terraform_files(self) -> list[str]:
        pass

    @property
    @abc.abstractmethod
    def backend_name(self) -> str:
        """
        The display name of the backend

        :return: a string of the backend name
        """
        pass

    @property
    @abc.abstractmethod
    def terraform_template_module(self) -> ModuleType:
        """
        The directory that stores this backend's templates

        :return: a pathlib.Path to the backend's templates'
        """
        pass

    @abc.abstractmethod
    def versions(self, bucket_name: str, remote_key: str = '') -> list[dict]:
        """
        List the versions of an object

        :param remote_key: the remote key (filename) to list
        :param bucket_name: the remote bucket name
        :return: a list of dictionaries containing 'version_id', 'last_modified', and 'size'
        """
        pass

    @abc.abstractmethod
    def store_object(self, local_file: Path, bucket_name: str,
                     remote_key: str, check_identical: bool) -> StoreOutcome:
        """
        Store an object remotely

        :param local_file: the local file to store
        :param bucket_name: the remote bucket name
        :param remote_key: the remote key (filename) of the object
        :param check_identical: whether to check if the last version is already the same as this version
        :return: a response enum StoreOutcome
        """
        pass

    @abc.abstractmethod
    def get_object(self, bucket_name: str, remote_key: str,
                   version_id: str) -> bytes | None:
        """
        Retrieve an object from the remote store as bytes

        :param bucket_name: the remote bucket name
        :param remote_key: the remote key (filename) of the object
        :param version_id: the version ID to fetch
        :return: the bytes of the retrieved object
        """
        pass

    @abc.abstractmethod
    def download_object(self, local_file: Path, bucket_name: str,
                        remote_key: str, version_id: str) -> bool:
        """
        Retrieve an object from the remote store and save it to a file

        :param local_file: the location to store the local file
        :param bucket_name: the remote bucket name
        :param remote_key: the remote key (filename) of the object
        :param version_id: the version ID to fetch
        :return: a true/false boolean of success
        """
        pass


class BackendFactory:
    @staticmethod
    def get_backend(backend_name: str = '',
                    raise_on_none: bool = False) -> AbstractBackend | None:
        """
        Return the active backend

        :param backend_name: the specific backend to return. Otherwise, uses the value of CARETAKER_BACKEND in Django settings.
        :param raise_on_none: whether to raise an exception if the backend isn't found
        :raises BackendNotFoundError: when the backend is not found and raise_on_none is set to True
        :return:
        """

        # see if there's a list of backends in the settings file
        # if there is, use it
        backends = ['caretaker.backend.backends.s3']

        if hasattr(settings, 'CARETAKER_BACKENDS') and \
                settings.CARETAKER_BACKENDS:
            backends = settings.CARETAKER_BACKENDS

        # get the backend name in this order:
        # 1. passed to function
        # 2. passed to settings.py
        if backend_name == '':
            backend_name = settings.CARETAKER_BACKEND

        if not backend_name or backend_name == '':
            backend_name = 'Amazon S3'

        # dynamically load modules in the backends space
        for full_package_name in backends:
            if full_package_name not in sys.modules:
                module = importlib.import_module(full_package_name)
                backend = module.get_backend()

                if backend.backend_name == backend_name:
                    return backend
            else:
                backend = sys.modules[full_package_name].get_backend()

                if backend.backend_name == backend_name:
                    return backend

        if raise_on_none:
            raise BackendNotFoundError

        return None


class BackendNotFoundError (Exception):
    pass



import logging
import tempfile
from io import StringIO
from pathlib import Path

from botocore.exceptions import ClientError
from django.conf import settings
from django.core.management import call_command

from caretaker.backend.abstract_backend import AbstractBackend, StoreOutcome
from caretaker.frontend.abstract_frontend import AbstractFrontend
from caretaker.utils import log, file
from caretaker.utils.zip import create_zip_file


def get_frontend():
    return DjangoFrontend()


class DjangoFrontend(AbstractFrontend):
    def __init__(self, logger: logging.Logger | None = None):
        super().__init__(logger)

        self.logger = log.get_logger('caretaker-django')

    @property
    def frontend_name(self) -> str:
        """
        The display name of the frontend

        :return: a string of the frontend name
        """
        return 'Django'

    @staticmethod
    def create_backup(output_directory: str, data_file: str = 'data.json',
                      archive_file: str = 'media.zip',
                      path_list: list | None = None) -> (Path | None,
                                                         Path | None):
        """
        Creates a set of local backup files

        :param output_directory: the output location
        :param data_file: the output data file (e.g. data.json)
        :param archive_file: the output archive file (e.g. media.zip)
        :param path_list: the list of paths to bundle in the zip
        :return: a 2-tuple of pathlib.Path objects to the data file and archive file
        """
        logger = log.get_logger('caretaker')

        if not output_directory:
            logger.error('No output directory specified')
            return None, None

        output_directory = file.normalize_path(output_directory)

        # create the directory if needed
        output_directory.mkdir(parents=True, exist_ok=True)

        # setup redirect so that we can pipe the output of dump data to
        # our output file
        buffer = StringIO()
        call_command('dumpdata', stdout=buffer)
        buffer.seek(0)

        with (output_directory / data_file).open('w') as out_file:
            out_file.write(buffer.read())
            logger.info('Wrote {}'.format(data_file))

        # now create a zip of the media directory and any others specified
        path_list = [] if not path_list else path_list
        path_list = list(set(path_list))

        if hasattr(settings, 'MEDIA_ROOT') and \
                settings.MEDIA_ROOT and settings.MEDIA_ROOT not in path_list:
            path_list.append(settings.MEDIA_ROOT)

        if hasattr(settings, 'ADDITIONAL_BACKUP_PATHS') \
                and settings.CARETAKER_ADDITIONAL_BACKUP_PATHS \
                and settings.CARETAKER_ADDITIONAL_BACKUP_PATHS not in path_list:
            path_list.extend(settings.CARETAKER_ADDITIONAL_BACKUP_PATHS)

        path_list_final = [Path(path).expanduser().resolve(strict=True)
                           for path in path_list]

        zip_file = create_zip_file(
            input_paths=path_list_final,
            output_file=Path(output_directory / archive_file)
        )

        logger.info('Wrote {} ({})'.format(archive_file, zip_file))

        return output_directory / data_file, zip_file

    @staticmethod
    def generate_terraform(output_directory: str,
                           backend: AbstractBackend) -> Path | None:
        """
        Generate a set of Terraform output files to provision an infrastructure

        :param output_directory: the output directory to write to
        :param backend: the backend to use
        :return: a path indicating where the Terraform files reside
        """
        logger = log.get_logger('caretaker')
        output_directory = file.normalize_path(output_directory)

        for filename in backend.terraform_files:
            terraform_file = file.get_package_file(backend=backend,
                                                   filename=filename)
            logger.info('Writing {} to {}'.format(filename, output_directory))
            file.output_terraform_file(
                output_directory=output_directory,
                terraform_file=terraform_file, file_name=filename)

        logger.info('Terraform files were written to {}'.format(
            output_directory))

        return output_directory

    @staticmethod
    def list_backups(remote_key: str, backend: AbstractBackend,
                     bucket_name: str) -> list[dict]:
        """
        Lists backups in the remote store

        :param remote_key: the remote key (filename)
        :param backend: the backend to use
        :param bucket_name: the name of the bucket/store
        :return: a list of dictionaries that contain the keys "last_modified", "version_id", and "size"
        """
        results = backend.versions(remote_key=remote_key,
                                   bucket_name=bucket_name)

        return results

    @staticmethod
    def pull_backup(backup_version: str, out_file: str, remote_key: str,
                    backend: AbstractBackend, bucket_name: str) -> Path | None:
        """
        Pull a backup object from the remote store

        :param backup_version: the version ID of the backup to pull
        :param out_file: the output file/download location
        :param remote_key: the remote key (filename)
        :param backend: the backend to use
        :param bucket_name: the name of the bucket/store
        :return: a pathlib.Path object pointing to the downloaded file or None
        """
        logger = log.get_logger('caretaker')

        out_file = file.normalize_path(out_file)

        download = backend.download_object(local_file=out_file,
                                           remote_key=remote_key,
                                           version_id=backup_version,
                                           bucket_name=bucket_name)

        try:
            if download:

                return out_file
            else:
                raise ClientError
        except ClientError:
            logger.error('Unable to download version {} of {} to {}'.format(
                backup_version,
                remote_key,
                out_file
            ))

            return None

    @staticmethod
    def push_backup(backup_local_file: str, remote_key: str,
                    backend: AbstractBackend, bucket_name: str) -> StoreOutcome:
        """
        Push a backup to the remote store

        :param backup_local_file: the local file to push
        :param remote_key: the remote key (filename)
        :param backend: the backend to use
        :param bucket_name: the name of the bucket/store
        :return: a StoreOutcome
        """
        logger = log.get_logger('caretaker')

        backup_local_file = file.normalize_path(backup_local_file)

        result = backend.store_object(remote_key=remote_key,
                                      bucket_name=bucket_name,
                                      local_file=backup_local_file,
                                      check_identical=True)

        match result:
            case StoreOutcome.STORED:
                logger.info('Stored backup.')
            case StoreOutcome.FAILED:
                logger.info('Failed to store backup.')
            case StoreOutcome.IDENTICAL:
                logger.info('Last version was identical.')

        return result

    @staticmethod
    def run_backup(output_directory: str, data_file: str = 'data.json',
                   archive_file: str = 'media.zip',
                   path_list: list | None = None,
                   backend: AbstractBackend | None = None,
                   bucket_name: str | None = None) -> (Path | None,
                                                       Path | None):
        """
        Creates a backup set and pushes it to the remote store

        :param output_directory: the output directory for the local backup set
        :param data_file: the output data file (e.g. data.json)
        :param archive_file: the output archive file (e.g. media.zip)
        :param path_list: the list of paths to bundle in the zip
        :param backend: the backend to use
        :param bucket_name: the name of the bucket/store
        :return: 2-tuple of pathlib.Path objects to the data file & archive file
        """
        logger = log.get_logger('caretaker')

        if not path_list:
            path_list = []

        if not output_directory:
            logger.error('No output directory specified')
            return None, None

        # set up a temporary directory
        with tempfile.TemporaryDirectory() as temporary_directory_name:
            # create a local backup set in this temporary directory
            json_file, zip_file = DjangoFrontend.create_backup(
                output_directory=temporary_directory_name,
                path_list=path_list
            )

            # push the data
            DjangoFrontend.push_backup(backup_local_file=json_file,
                                       remote_key=data_file,
                                       backend=backend, bucket_name=bucket_name)
            DjangoFrontend.push_backup(backup_local_file=zip_file,
                                       remote_key=archive_file,
                                       backend=backend, bucket_name=bucket_name)

            logger.info('Pushed backups to remote store')
            return json_file, archive_file



import humanize
from django.conf import settings
from django.contrib.admin.views.decorators import staff_member_required
from django.http import StreamingHttpResponse, HttpRequest, HttpResponse
from django.shortcuts import render

from caretaker.backend.abstract_backend import BackendFactory, AbstractBackend


@staff_member_required
def list_backups(request: HttpRequest) -> HttpResponse:
    """
    A Django view showing the list of objects

    :param request: the HttpRequest object
    :return: an HttpResponse
    """
    backend = BackendFactory.get_backend()

    sql_versions = _fetch_versions(backend, 'data.json')
    data_versions = _fetch_versions(backend, 'media.zip')

    template = 'backup_list.html'

    context = {'sql_versions': sql_versions,
               'data_versions': data_versions,
               }

    return render(
        request,
        template,
        context,
    )


@staff_member_required
def download_backup(request: HttpRequest, backup_type: str, version_id: str) \
        -> StreamingHttpResponse:
    """
    A view that allows the user to download a backup

    :param request: the HttpRequest object
    :param backup_type: the type of backup ('sql' or 'media.zip')
    :param version_id: the version ID to download
    :return: a StreamingHttpResponse
    """
    backend = BackendFactory.get_backend()

    if backup_type == 'sql':
        key = 'data.json'
    else:
        key = 'media.zip'

    response_object = backend.get_object(
        bucket_name=settings.CARETAKER_BACKUP_BUCKET,
        remote_key=key, version_id=version_id)

    resp = StreamingHttpResponse(streaming_content=response_object)
    resp['Content-Disposition'] = 'attachment; ' \
                                  'filename="{}-{}"'.format(version_id, key)

    return resp


def _fetch_versions(backend: AbstractBackend, key) -> list[dict]:
    """
    Fetches the version list for a remote key from the backend

    :param backend: the backend to use
    :param key: the remote key (filename)
    :return: a list of dictionaries containing 'version_id',
        'last_modified', and 'size'
    """
    versions = backend.versions(
        bucket_name=settings.CARETAKER_BACKUP_BUCKET,
        remote_key=key)

    sql_versions = []

    for item in versions:
        sql_versions.append({'last_modified': item['last_modified'],
                             'version_id': item['version_id'],
                             'size': humanize.naturalsize(item['size'])})

    return sql_versions



import tempfile
from pathlib import Path

from caretaker.tests.frontend.django.backend.s3.caretaker_test import \
    AbstractDjangoS3Test
from moto import mock_s3


@mock_s3
class TestTerraformOutputDjangoS3(AbstractDjangoS3Test):
    def setUp(self):
        self.logger.info('Setup for Terraform test')

        self.create_bucket()

    def tearDown(self):
        self.logger.info('Teardown Terraform test')
        pass

    def test(self):
        self.logger.info('Running Terraform test')
        with tempfile.TemporaryDirectory() as temporary_directory_name:
            self.frontend.generate_terraform(
                output_directory=temporary_directory_name,
                backend=self.backend
            )

            self.assertTrue(
                (Path(temporary_directory_name) / 'main.tf').exists())
            self.assertTrue(
                (Path(temporary_directory_name) / 'output.tf').exists())



import abc
import importlib
import logging
import sys
from pathlib import Path

from django.conf import settings

from caretaker.backend.abstract_backend import AbstractBackend, \
    BackendFactory, StoreOutcome


class AbstractFrontend(metaclass=abc.ABCMeta):

    @abc.abstractmethod
    def __init__(self, logger: logging.Logger | None = None):
        self.logger = logger

    @property
    @abc.abstractmethod
    def frontend_name(self) -> str:
        """
        The display name of the frontend

        :return: a string of the frontend name
        """
        pass

    @staticmethod
    @abc.abstractmethod
    def create_backup(output_directory: str, data_file: str = 'data.json',
                      archive_file: str = 'media.zip',
                      path_list: list | None = None) -> (Path | None,
                                                         Path | None):
        """
        Creates a set of local backup files

        :param output_directory: the output location
        :param data_file: the output data file (e.g. data.json)
        :param archive_file: the output archive file (e.g. media.zip)
        :param path_list: the list of paths to bundle in the zip
        :return: a 2-tuple of pathlib.Path objects to the data file and archive file
        """
        pass

    @staticmethod
    @abc.abstractmethod
    def generate_terraform(output_directory: str,
                           backend: AbstractBackend) -> Path | None:
        """
        Generate a set of Terraform output files to provision an infrastructure

        :param output_directory: the output directory to write to
        :param backend: the backend to use
        :return: a path indicating where the Terraform files reside
        """
        pass

    @staticmethod
    @abc.abstractmethod
    def list_backups(remote_key: str, backend: AbstractBackend,
                     bucket_name: str) -> list[dict]:
        """
        Lists backups in the remote store

        :param remote_key: the remote key (filename)
        :param backend: the backend to use
        :param bucket_name: the name of the bucket/store
        :return: a list of dictionaries that contain the keys "last_modified", "version_id", and "size"
        """
        pass

    @staticmethod
    @abc.abstractmethod
    def pull_backup(backup_version: str, out_file: str, remote_key: str,
                    backend: AbstractBackend, bucket_name: str) -> Path | None:
        """
        Pull a backup object from the remote store

        :param backup_version: the version ID of the backup to pull
        :param out_file: the output file/download location
        :param remote_key: the remote key (filename)
        :param backend: the backend to use
        :param bucket_name: the name of the bucket/store
        :return: a pathlib.Path object pointing to the downloaded file or None
        """

        pass

    @staticmethod
    @abc.abstractmethod
    def push_backup(backup_local_file: str, remote_key: str,
                    backend: AbstractBackend, bucket_name: str) -> StoreOutcome:
        """
        Push a backup to the remote store

        :param backup_local_file: the local file to push
        :param remote_key: the remote key (filename)
        :param backend: the backend to use
        :param bucket_name: the name of the bucket/store
        :return: a StoreOutcome
        """
        pass

    @staticmethod
    @abc.abstractmethod
    def run_backup(output_directory: str, data_file: str = 'data.json',
                   archive_file: str = 'media.zip',
                   path_list: list | None = None,
                   backend: AbstractBackend | None = None,
                   bucket_name: str | None = None) -> (Path | None,
                                                       Path | None):
        """
        Creates a backup set and pushes it to the remote store

        :param output_directory: the output directory for the local backup set
        :param data_file: the output data file (e.g. data.json)
        :param archive_file: the output archive file (e.g. media.zip)
        :param path_list: the list of paths to bundle in the zip
        :param backend: the backend to use
        :param bucket_name: the name of the bucket/store
        :return: 2-tuple of pathlib.Path objects to the data file & archive file
        """
        pass


class FrontendNotFoundError (Exception):
    pass


class FrontendFactory:
    @staticmethod
    def get_frontend(frontend_name: str = '',
                     raise_on_none: bool = False) -> AbstractFrontend | None:
        """
        Return the active frontend

        :param frontend_name: the specific frontend to return. Otherwise, uses the value of CARETAKER_FRONTEND in Django settings.
        :param raise_on_none: whether to raise an exception if the frontend isn't found
        :raises FrontendNotFoundError: when the frontend is not found and raise_on_none is set to True
        :return:
        """

        # see if there's a list of backends in the settings file
        # if there is, use it
        frontends = ['caretaker.frontend.frontends.django']

        if hasattr(settings, 'CARETAKER_FRONTENDS') and \
                settings.CARETAKER_FRONTENDS:
            frontends = settings.CARETAKER_FRONTENDS

        # get the backend name in this order:
        # 1. passed to function
        # 2. passed to settings.py
        if frontend_name == '':
            frontend_name = settings.CARETAKER_FRONTEND

        # set the default frontend to Django if we're still not found
        if not frontend_name or frontend_name == '':
            frontend_name = 'Django'

        # dynamically load modules in the backends space
        for full_package_name in frontends:
            if full_package_name not in sys.modules:
                module = importlib.import_module(full_package_name)
                frontend = module.get_frontend()

                if frontend.frontend_name == frontend_name:
                    return frontend
            else:
                frontend = sys.modules[full_package_name].get_frontend()

                if frontend.frontend_name == frontend_name:
                    return frontend

        if raise_on_none:
            raise FrontendNotFoundError

        return None

    @staticmethod
    def get_frontend_and_backend(
            frontend_name: str = '',
            backend_name: str = '',
            raise_on_none: bool = False) -> (AbstractFrontend | None,
                                             AbstractBackend | None):
        """
        Return the active frontend and backend

        :param frontend_name: the name of the frontend
        :param backend_name: the name of the backend
        :param raise_on_none: whether to raise exceptions if no backend is found
        :return: 2-tuple of a frontend and backend
        """

        frontend = FrontendFactory.get_frontend(frontend_name,
                                                raise_on_none=raise_on_none)
        backend = BackendFactory.get_backend(backend_name,
                                             raise_on_none=raise_on_none)

        return frontend, backend



from pathlib import Path
from zipfile import ZipFile, ZIP_DEFLATED
from caretaker.utils import file as file_util


def create_zip_file(input_paths: list, output_file: Path) -> Path:
    """
    Create a zip file that stores all input paths inside

    :param input_paths: a list of input directories
    :param output_file: the output file to write
    :return: a pathlib.Path object pointing to the zip
    """
    with ZipFile(file_util.normalize_path(output_file),
                 'w', ZIP_DEFLATED) as zf:
        for directory in input_paths:
            for file in directory.rglob('*'):
                zf.write(file, file.relative_to(directory.parent))

    return Path(output_file)



import importlib.resources as pkg_resources
from pathlib import Path
from typing import TextIO

from django.conf import settings
from django.template import Template, Context

from caretaker.backend.abstract_backend import AbstractBackend


def normalize_path(path: str | Path) -> Path:
    """
    Normalize paths by a standard method

    :param path: the location on disk
    :return: a pathlib.Path object
    """
    return Path(path).expanduser()


def get_package_file(filename: str, backend: AbstractBackend) -> TextIO:

    return pkg_resources.open_text(backend.terraform_template_module,
                                   filename)


def output_terraform_file(output_directory: Path, terraform_file: TextIO,
                          file_name: str) -> Path | None:
    """
    Writes a Terraform file to an output directory

    :param output_directory: the output directory
    :param terraform_file: the input file to template and copy
    :param file_name: the output filename
    :return: a pathlib.Path of the output file
    """

    with terraform_file as in_file:
        # render the terraform file into a template
        t = Template(in_file.read())
        c = Context({"bucket_name": settings.CARETAKER_BACKUP_BUCKET})
        rendered = t.render(c)

        # create the file structure
        output_directory.mkdir(parents=True, exist_ok=True)
        output_file = (output_directory / file_name)

        # write the terraform file to this directory
        with output_file.open('w') as out_file:
            out_file.write(rendered)

        return output_file



from setuptools import setup

# read the contents of your README file
from pathlib import Path
this_directory = Path(__file__).parent

long_description = (this_directory / "README.md").read_text()

setup(name='django-caretaker', long_description=long_description,
      long_description_content_type='text/markdown')



import logging
from rich.logging import RichHandler


def get_logger(namespace: str = 'root', level: int = logging.INFO) \
        -> logging.Logger:
    """
    Get a Rich logger for the provided namespace

    :param namespace: the namespace identifier
    :param level: the level of logging to indicate
    :return: a logging.Logger object
    """
    format_string = "%(message)s"
    logging.basicConfig(
        level=level, format=format_string,
        datefmt="[%X]", handlers=[RichHandler()]
    )

    logger = logging.getLogger(namespace)
    logger.setLevel(level)

    return logger



import tempfile

import django
from moto import mock_s3

from caretaker.backend.abstract_backend import StoreOutcome
from caretaker.tests.frontend.django.backend.s3.caretaker_test import \
    AbstractDjangoS3Test
from caretaker.tests.utils import upload_temporary_file


@mock_s3
class TestCreateBackupDjangoS3(AbstractDjangoS3Test):
    def setUp(self):
        self.logger.info('Setup for create_backup')

        self.create_bucket()

        django.setup()

    def tearDown(self):
        self.logger.info('Teardown for create_backup')
        pass

    def test(self):
        self.logger.info('Testing create_backup')

        with tempfile.TemporaryDirectory() as temporary_directory_name:
            # set up a temporary file
            self.logger.info('Uploading temporary file')
            result, temporary_file = upload_temporary_file(
                test_class=self,
                temporary_directory_name=temporary_directory_name,
                contents=self.test_contents)

            self.assertTrue(result == StoreOutcome.STORED)

            # create a backup record including this directory
            json_file, data_file = self.frontend.create_backup(
                output_directory=temporary_directory_name,
                path_list=[temporary_directory_name]
            )

            self.assertTrue(json_file.exists())
            self.assertTrue(data_file.exists())
